package org.brynhild.controller;

import org.brynhild.common.transfer.dto.Result;
import org.brynhild.common.utils.StringUtil;
import org.brynhild.office.action.SortConstrain;
import org.brynhild.office.csv.CSVPrepare;
import org.brynhild.office.csv.CSVReader;
import org.brynhild.office.csv.Column;
import org.brynhild.office.csv.Row;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Predicate;

@RestController
@RequestMapping("/c1")
public class Chapter1Controller {
    @GetMapping("/f1")
    public Result C1F1() {
        try {
            CSVReader reader = new CSVReader("dataSet" + File.separator + "中国各省人口及累计确诊数量.csv");
            final Column date = reader.getColumn("date");
            final Column confirm = reader.getColumn("confirm");
            final Column dead = reader.getColumn("dead");
            final Column heal = reader.getColumn("heal");

            final Integer[] accumulate = {0, 0, 0};
            final List<String> dateList = reader.getColumnData(date);
            final List<Integer> confirmList = reader.getColumnData(confirm, Integer.class);
            final List<Integer> accumulateConfirm = new ArrayList<>();
            final List<Integer> deadList = reader.getColumnData(dead, Integer.class);
            final List<Integer> accumulateDead = new ArrayList<>();
            final List<Integer> healList = reader.getColumnData(heal, Integer.class);
            final List<Integer> accumulateHeal = new ArrayList<>();
            for (int i = 0; i < dateList.size(); i++) {
                final int dailyConfirm = confirmList.get(i);
                final int dailyDead = deadList.get(i);
                final int dailyHeal = healList.get(i);
                accumulate[0] += dailyConfirm;
                accumulate[1] += dailyDead;
                accumulate[2] += dailyHeal;
                accumulateConfirm.add(accumulate[0]);
                accumulateDead.add(accumulate[1]);
                accumulateHeal.add(accumulate[2]);
            }

            Map<String, Object> ret = new HashMap<String, Object>() {{
                put("date", dateList);
                put("confirm", confirmList);
                put("accumulateConfirm", accumulateConfirm);
                put("dead", deadList);
                put("accumulateDead", accumulateDead);
                put("heal", healList);
                put("accumulateHeal", accumulateHeal);
                put("title", "中国各省人口及累计确诊数量");
            }};
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f2")
    public Result C1F2() {
        try {
            CSVReader reader = new CSVReader("dataSet" + File.separator + "湖北省每日分城市累计确诊.csv");
            final Column city = reader.getColumn("city");
            final Column cityCode = reader.getColumn("cityCode");

            final Column confirmed = reader.getColumn("confirmed");
            final Column heal = reader.getColumn("cured");
            final Column dead = reader.getColumn("dead");
            final Column date = reader.getColumn("date");

            Predicate<Row> predicate = row -> StringUtil.isEmpty(row.get(city)) &&
                    StringUtil.isEmpty(row.get(cityCode)) &&
                    !StringUtil.isEmpty(row.get(confirmed));
            reader=new CSVPrepare(reader)
                    .addPredicate(predicate)
                    .compute();

            final Integer[] accumulate = {0, 0, 0};
            final List<String> dateList = reader.getColumnData(date);
            final List<Integer> confirmList = reader.getColumnData(confirmed, Integer.class);
            final List<Integer> accumulateConfirm = new ArrayList<>();
            final List<Integer> deadList = reader.getColumnData(dead, Integer.class);
            final List<Integer> accumulateDead = new ArrayList<>();
            final List<Integer> healList = reader.getColumnData(heal, Integer.class);
            final List<Integer> accumulateHeal = new ArrayList<>();
            for (int i = 0; i < dateList.size(); i++) {
                final int dailyConfirm = confirmList.get(i);
                final int dailyDead = deadList.get(i);
                final int dailyHeal = healList.get(i);
                accumulate[0] += dailyConfirm;
                accumulate[1] += dailyDead;
                accumulate[2] += dailyHeal;
                accumulateConfirm.add(accumulate[0]);
                accumulateDead.add(accumulate[1]);
                accumulateHeal.add(accumulate[2]);
            }

            Map<String, Object> ret = new HashMap<String, Object>() {{
                put("date", dateList);
                put("confirm", confirmList);
                put("accumulateConfirm", accumulateConfirm);
                put("dead", deadList);
                put("accumulateDead", accumulateDead);
                put("heal", healList);
                put("accumulateHeal", accumulateHeal);
                put("title", "湖北每日新增确诊新冠病例");
            }};

            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f3")
    public Result C1F3() {
        try {
            CSVReader reader = new CSVReader("dataSet" + File.separator + "世界截止目前累计确诊.csv");
            final Column confirm = reader.getColumn("confirm");
            final Column heal = reader.getColumn("heal");
            final Column dead = reader.getColumn("dead");


            final CSVReader confirmReader = new CSVPrepare(reader)
                    .addSortRule(confirm, Integer.class, SortConstrain.GREATER)
                    .compute();
            final CSVReader healReader = new CSVPrepare(reader)
                    .addSortRule(heal, Integer.class, SortConstrain.GREATER)
                    .compute();
            final CSVReader deadReader = new CSVPrepare(reader)
                    .addSortRule(dead, Integer.class, SortConstrain.GREATER)
                    .compute();
            Map<String, Object> ret = new HashMap<String, Object>() {{
                put("confirm", confirmReader.getColumnData(confirm));
                put("confirmCounty", confirmReader.getColumnData(confirmReader.getColumn(0)));
                put("heal", healReader.getColumnData(heal));
                put("healCounty", healReader.getColumnData(healReader.getColumn(0)));
                put("dead", deadReader.getColumnData(dead));
                put("deadCounty", deadReader.getColumnData(deadReader.getColumn(0)));
                put("title", "世界上确诊、治愈、死亡人数的国家排行");
            }};
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f4")
    public Result C1F4() {
        try {
            CSVReader reader = new CSVReader("dataSet" + File.separator + "中国-各年龄段累计确诊占比.csv");
            final Column age = reader.getColumn(0);
            final Column women = reader.getColumn(1);
            final Column men = reader.getColumn(2);
            Map<String, Object> ret = new HashMap<String, Object>() {{
                put("women", new HashMap<String, List<String>>() {{
                    put("age", reader.getColumnData(age));
                    put("rate", reader.getColumnData(women));
                }});
                put("men", new HashMap<String, Object>() {{
                    put("age", reader.getColumnData(age));
                    put("rate", reader.getColumnData(men));
                }});
            }};
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f5")
    public Result C1F5() {
        try {
            Map<String, Object> ret = new HashMap<String, Object>() {{
                put("city", "武汉市、黄石市、十堰市、宜昌市、襄阳市、鄂州市、荆门市、孝感市、荆州市、黄冈市、咸宁市、随州市、恩施".split("、"));
                put("popu", "1060,244,338,411,565,107,290,491,564,740,253,221,336".split(","));
                put("confirm", "50340,1015,672,931,1175,1394,928,3518,1580,2907,836,1307,252".split(","));
            }};
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    public Result empty() {
        try {
            CSVReader reader = new CSVReader("dataSet" + File.separator + "");
            Map<String, Object> ret = new HashMap<String, Object>() {{

            }};
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

}
