package org.brynhild.controller;

import com.kennycason.kumo.CollisionMode;
import com.kennycason.kumo.WordCloud;
import com.kennycason.kumo.WordFrequency;
import com.kennycason.kumo.bg.CircleBackground;
import com.kennycason.kumo.font.KumoFont;
import com.kennycason.kumo.font.scale.SqrtFontScalar;
import com.kennycason.kumo.nlp.FrequencyAnalyzer;
import com.kennycason.kumo.nlp.tokenizers.ChineseWordTokenizer;
import com.kennycason.kumo.palette.LinearGradientColorPalette;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.awt.*;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.util.List;
import java.util.*;

@RestController
@RequestMapping("/c2")
public class Chapter2Controller {

    private static void createWordCountPic(HttpServletRequest request, HttpServletResponse response) throws IOException {
        FrequencyAnalyzer frequencyAnalyzer = new FrequencyAnalyzer();
        frequencyAnalyzer.setWordFrequenciesToReturn(5);
        frequencyAnalyzer.setMinWordLength(1);
        frequencyAnalyzer.setWordTokenizer(new ChineseWordTokenizer());
        File file = new File("dataSet" + File.separator + "ciyun.txt");

        Scanner scanner = new Scanner(Files.newInputStream(file.toPath()));
        final String line = scanner.nextLine();
        final String[] words = line.split(",");
        Map<String, Integer> map = new HashMap<>();
        for (String word : words) {
            map.putIfAbsent(word, 0);
            map.put(word, map.get(word) + 1);
        }
        List<WordFrequency> wordFrequencies = new ArrayList<>();
        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            if (entry.getValue() < 10) {
                continue;
            }
            wordFrequencies.add(new WordFrequency(entry.getKey(), entry.getValue()));
        }
        //加入分词并随机生成权重，每次生成得图片都不一样
        //此处不设置会出现中文乱码
        java.awt.Font font = new java.awt.Font("STSong-Light", Font.ITALIC, 18);
        //设置图片分辨率
        Dimension dimension = new Dimension(500, 500);
        //此处的设置采用内置常量即可，生成词云对象
        WordCloud wordCloud = new WordCloud(dimension, CollisionMode.PIXEL_PERFECT);
        //设置边界及字体
        wordCloud.setPadding(2);
        //因为我这边是生成一个圆形,这边设置圆的半径
        wordCloud.setBackground(new CircleBackground(255));
        wordCloud.setFontScalar(new SqrtFontScalar(20, 60));
        //设置词云显示的三种颜色，越靠前设置表示词频越高的词语的颜色
        wordCloud.setColorPalette(new LinearGradientColorPalette(Color.RED, Color.BLUE, Color.GREEN, 30, 30));
        wordCloud.setKumoFont(new KumoFont(font));
        wordCloud.setBackgroundColor(new Color(255, 255, 255));
        wordCloud.build(wordFrequencies);
        response.setContentType("image/jpeg");

        wordCloud.writeToFile("ciyun.png");
        final File imageFile = new File("ciyun.png");
        final ServletOutputStream stream = response.getOutputStream();
        byte[] buffer = new byte[1024];
        final InputStream imageStream = Files.newInputStream(imageFile.toPath());
        int count = 0;
        while ((count = imageStream.read(buffer)) != -1) {
            stream.write(buffer, 0, count);
        }
        stream.flush();
        imageStream.close();
    }

    @GetMapping("/f1")
    public void C2F1(HttpServletRequest request, HttpServletResponse response) {
        try {
            createWordCountPic(request, response);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}
