package org.brynhild.controller;

import org.brynhild.common.transfer.dto.Result;
import org.brynhild.common.utils.StringUtil;
import org.brynhild.common.utils.TimeUtil;
import org.brynhild.office.csv.CSVPrepare;
import org.brynhild.office.csv.CSVReader;
import org.brynhild.office.csv.CSVWriter;
import org.brynhild.office.csv.Column;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.sql.Time;
import java.text.SimpleDateFormat;
import java.util.*;

@RestController
@RequestMapping("/c3")
public class Chapter3Controller {

    private double getRate(Integer current, Integer before) {
        if(current<1){
            return 0.01;
        }
        return Math.max(0.01,1.0 * (current - before) / current);
    }

    @GetMapping("/f1")
    public Result C3F1() {
        try {
//            CSVReader reader = new CSVReader("dataSet" + File.separator + "Wuhan-2019-nCoV.csv");
            CSVReader reader = new CSVReader("dataSet" + File.separator + "c3f1.csv");
            String[] cityList = new String[]{"武汉市", "黄石市", "十堰市", "宜昌市", "襄阳市", "鄂州市", "荆门市",
                    "孝感市", "荆州市", "黄冈市", "咸宁市", "随州市", "恩施土家族苗族自治州",
                    "仙桃市", "潜江市", "天门市"};
            final List<String> dateList = TimeUtil.generateDateSection(
                    TimeUtil.parseDate("2020-01-31"),
                    TimeUtil.parseDate("2020-02-28"),
                    4);
            final Column confirmed = reader.getColumn("confirmed");
            final Column city = reader.getColumn("city");
            final Column date = reader.getColumn("date");

            final CSVReader dateConstrain = new CSVPrepare(reader)
                    .addPredicate(row -> StringUtil.anyMatch(row.get(date), dateList))
                    .addPredicate(row -> StringUtil.anyMatch(row.get(city),cityList))
                    .compute();
//            new CSVWriter(dateConstrain,"dataSet"+File.separator+"c3f1.csv").write().finish();
            Map<String, Object> ret = new HashMap<>();
            List<Map<String, Object>> rateList = new ArrayList<>();

            for (String cityString : cityList) {
                final List<Integer> cityData = dateConstrain.getColumnData(confirmed, Integer.class, row -> row.get(city).equals(cityString));
                final List<Double> rateData = new ArrayList<>();
                for (int i = 1; i < cityData.size(); i++) {
                    rateData.add(getRate(cityData.get(i), cityData.get(i - 1)));
                }
                rateList.add(new HashMap<String, Object>() {{
                    put("city", cityString);
                    put("rate", rateData);
                }});
            }
            ret.put("info", rateList);
            dateList.remove(0);
            ret.put("dateList", dateList);
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f2")
    public Result C3F2() {
        try {
//            CSVReader reader = new CSVReader("dataSet" + File.separator + "Wuhan-2019-nCoV.csv");
            CSVReader reader = new CSVReader("dataSet" + File.separator + "c3f2.csv");
            String[] provinceList = new String[]{"北京市", "天津市", "河北省", "山西省", "内蒙古自治区", "辽宁省", "吉林省",
                    "黑龙江省", "上海市", "江苏省", "浙江省", "安徽省", "福建省", "江西省", "山东省",
                    "河南省", "湖北省", "湖南省", "广东省", "广西壮族自治区", "海南省", "重庆市", "四川省",
                    "贵州省", "云南省", "西藏自治区", "陕西省", "甘肃省", "青海省", "宁夏回族自治区", "新疆维吾尔自治区",
                    "台湾省", "香港特别行政区", "澳门特别行政区"};
            final List<String> dateList = TimeUtil.generateDateSection(
                    TimeUtil.parseDate("2020-02-01"),
                    TimeUtil.parseDate("2020-04-28"),
                    4);
            final Column confirmed = reader.getColumn("confirmed");
            final Column province = reader.getColumn("province");
            final Column date = reader.getColumn("date");
            final Column city = reader.getColumn("city");

            final CSVReader dateConstrain = new CSVPrepare(reader)
                    .addPredicate(row -> StringUtil.anyMatch(row.get(date), dateList))
                    .addPredicate(row -> StringUtil.anyMatch(row.get(province), provinceList))
                    .addPredicate(row -> StringUtil.isEmpty(row.get(city)))
                    .compute();
//            new CSVWriter(dateConstrain,"dataSet"+File.separator+"c3f2.csv").write().finish();
            Map<String, Object> ret = new HashMap<String,Object>(){{
                List<Map<String, Object>> rateList = new ArrayList<>();
                for (String provinceString : provinceList) {
                    final List<Integer> cityData = dateConstrain.getColumnData(confirmed, Integer.class, row -> row.get(province).equals(provinceString));
                    final List<Double> rateData = new ArrayList<>();
                    for (int i = 1; i < cityData.size(); i++) {
                        rateData.add(getRate(cityData.get(i), cityData.get(i - 1)));
                    }
                    rateList.add(new HashMap<String, Object>() {{
                        put("city", provinceString);
                        put("rate", rateData);
                    }});
                }
                put("info", rateList);
                dateList.remove(0);
                put("dateList", dateList);
            }};

            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }

    @GetMapping("/f3")
    public Result C3F3(){
        try {
//            CSVReader reader = new CSVReader("dataSet" + File.separator + "Wuhan-2019-nCoV.csv");
            CSVReader reader = new CSVReader("dataSet" + File.separator + "c3f3.csv");
            String[] countryList = new String[]{"阿联酋","澳大利亚","比利时","加拿大","中国","德国","西班牙","芬兰",
                    "法国","英国","印度","意大利","日本","柬埔寨","韩国","斯里兰卡","马来西亚",
                    "尼泊尔","菲律宾","俄罗斯","瑞典","新加坡","泰国","美国","越南"};
            final List<String> dateList = TimeUtil.generateDateSection(
                    TimeUtil.parseDate("2020-01-01"),
                    TimeUtil.parseDate("2020-06-10"),
                    9);
            final Column confirmed = reader.getColumn("confirmed");
            final Column country = reader.getColumn("country");
            final Column province = reader.getColumn("province");
            final Column city = reader.getColumn("city");
            final Column date = reader.getColumn("date");

            final CSVReader dateConstrain = new CSVPrepare(reader)
                    .addPredicate(row -> StringUtil.anyMatch(row.get(date), dateList))
                    .addPredicate(row -> StringUtil.anyMatch(row.get(country), countryList))
                    .addPredicate(row -> StringUtil.isEmpty(row.get(city)))
                    .addPredicate(row->StringUtil.isEmpty(row.get(province)))
                    .compute();

//            new CSVWriter(dateConstrain,"dataSet"+File.separator+"c3f3.csv").write().finish();
            Map<String, Object> ret = new HashMap<>();
            List<Map<String, Object>> rateList = new ArrayList<>();

            int count=dateList.size();
            for (String countryString : countryList) {
                final List<Integer> cityData = dateConstrain.getColumnData(confirmed, Integer.class, row -> row.get(country).equals(countryString));
                final List<Double> rateData = new ArrayList<>();
                for (int i = 1; i < cityData.size(); i++) {
                    rateData.add(getRate(cityData.get(i), cityData.get(i - 1)));
                }
                while(rateData.size()<dateList.size()-1){
                    rateData.add(0,0.01);
                }
                rateList.add(new HashMap<String, Object>() {{
                    put("city", countryString);
                    put("rate", rateData);
                }});
            }
            ret.put("info", rateList);
            dateList.remove(0);
            ret.put("dateList", dateList);
            return new Result(true, 200, "查询成功", ret);
        } catch (Exception e) {
            e.printStackTrace();
            return new Result(false, 500, "发生错误", e.getMessage());
        }
    }
}
